@@ -14,7 +14,8 @@ import pandas as pd
 import pysam
 
 from pomoxis import bio
-
+import pomoxis.util
+from pomoxis.summary_from_stats import qscore
 
 class AverageScore:
     """Keep track of a simple average of inputs."""
@@ -48,23 +49,31 @@ def get_fraction_correct(counts):
     """Calculate fraction correct vs length from counts."""
     corr_res = []
     for base, c in counts.items():
-        for rlen, sub_counts in c.items():
-            corr_res.append({
-                'qbase': base,
-                'rlen': rlen,
-                'n': sum(sub_counts.values()),
-                'n_corr': sub_counts[rlen],
-                'fr_corr': sub_counts[rlen] / max(sum(sub_counts.values()), 1)
-                })
+        if base != 'length':
+            for rlen, sub_counts in c.items():
+                corr_res.append({
+                    'qbase': base,
+                    'rlen': rlen,
+                    'n': sum(sub_counts.values()),
+                    'n_corr': sub_counts[rlen],
+                    'fr_corr': sub_counts[rlen] / max(sum(sub_counts.values()), 1)
+                    })
     df = pd.DataFrame(corr_res)
     df_fr = df.pivot(index='rlen', columns='qbase', values='fr_corr').reset_index()
     df_n = df.pivot(index='rlen', columns='qbase', values='n').reset_index()
+    df_n[np.isnan(df_n)] = 0
     df = pd.merge(df_fr, df_n, on='rlen', suffixes=['', '_n'])
+    for b in ['A', 'T', 'G', 'C']:
+        if b not in df:
+            df.loc[:, b] = np.NaN
+            df.loc[:, b + '_n'] = 0
     for b, c in [('A', 'T'), ('G', 'C')]:
         # get total count and weighted-fraction correct for AT and GC
         sum_n_col = b + c + '_n'
-        df[sum_n_col] = df[b + '_n'] + df[c + '_n']
-        df[b + c] = ((df[b + '_n'] * df[b]) + (df[c + '_n'] * df[c])) / df[sum_n_col]
+        df[sum_n_col] = df[b + '_n'].add(df[c + '_n'], fill_value=0)
+        df[b + c] = sum([df[a + '_n'].mul(df[a], fill_value=0) for a in [b, c]]) / df[sum_n_col]
+    df['ATGC_n'] = df.loc[:, ('A_n', 'T_n', 'G_n', 'C_n')].sum(axis=1)
+    df['ATGC'] = sum([df[b + '_n'].mul(df[b], fill_value=0) for b in ('AT', 'GC')]) / df['ATGC_n']
     return df
 
 
@@ -77,18 +86,20 @@ def plot_fraction_correct(df, fname, cols=('A', 'T', 'G', 'C')):
     ax.set_xlabel('Homopolymer length')
     ax.legend(frameon=False)
     fig.savefig(fname)
+    plt.close(fig)
 
 
 def get_relative_lengths(counts):
     rows = []
     for q_base, c in counts.items():
-        for ref_len, qc in c.items():
-            data = {'ref_len': ref_len, 'q_base': q_base}
-            for query_len, count in qc.items():
-                rows.append(data.copy())
-                rows[-1]['query_len'] = query_len
-                rows[-1]['count'] = count
-                rows[-1]['rel_len'] = query_len - ref_len
+        if q_base != 'length':
+            for ref_len, qc in c.items():
+                data = {'ref_len': ref_len, 'q_base': q_base}
+                for query_len, count in qc.items():
+                    rows.append(data.copy())
+                    rows[-1]['query_len'] = query_len
+                    rows[-1]['count'] = count
+                    rows[-1]['rel_len'] = query_len - ref_len
 
     return pd.DataFrame(rows)
 
@@ -111,7 +122,7 @@ def plot_relative_lengths(data, fname):
     cols = sorted(data['q_base'].unique())
     fig, axes = plt.subplots(
         ncols=len(cols), nrows=len(rows), sharex=True,
-        figsize=(4 * len(cols), 2 * len(rows)))
+        figsize=(4 * len(cols), 2 * len(rows)), squeeze=False)
     for rl, rl_df in data.groupby(['ref_len']):
         i = rows.index(rl)
         for qb, qb_df in rl_df.groupby('q_base'):
@@ -129,6 +140,85 @@ def plot_relative_lengths(data, fname):
         ax.set_ylabel('Counts')
     fig.tight_layout()
     fig.savefig(fname)
+    plt.close(fig)
+
+
+def get_heatmap(data, data_comp):
+    """Transform relative length data into a 2d array of called length vs true length."""
+    df = data.pivot(index='query_len', columns=['q_base', 'ref_len'],
+        values='count').sort_index(axis=0).sort_index(axis=1)
+    column_reindex = pd.MultiIndex.from_product((['A','T','G','C'],
+            range(df.columns.levels[1].min(), df.columns.levels[1].max() + 1)))
+    df = df.reindex(range(df.index.min(), df.index.max() + 1))
+    df[np.isnan(df)] = 0
+    df = df.reindex(column_reindex, axis=1)
+
+    df_comp = data_comp.copy()
+    df_comp['query_len'] = df_comp['ref_len'] + df_comp['rel_len']
+    df_comp = df_comp.pivot(index='query_len', columns=['q_base', 'ref_len'],
+        values='count').sort_index(axis=0).sort_index(axis=1)
+    column_reindex = pd.MultiIndex.from_product((df_comp.columns.levels[0],
+            range(df_comp.columns.levels[1].min(), df_comp.columns.levels[1].max() + 1)),
+            names=['q_base', 'ref_len'])
+    df_comp = df_comp.reindex(range(df_comp.index.min(), df_comp.index.max() + 1))
+    df_comp[np.isnan(df_comp)] = 0
+    df_comp = df_comp.reindex(column_reindex, axis=1)
+
+    df_all = df_comp.sum(level='ref_len', axis=1)
+    df_all.columns = pd.MultiIndex.from_product((['ATGC',], df_all.columns), 
+        names=['q_base', 'ref_len'])
+
+    df = pd.concat((df, df_comp, df_all), axis=1)
+    df[np.isnan(df)] = 0
+    return df
+
+
+def plot_heatmap(df, fname, cols=('A', 'T', 'G', 'C')):
+    """Plot heatmaps of called length vs true length."""
+    fig, axes = plt.subplots(1, len(cols), figsize=(6 * len(cols), 4),
+            squeeze=False)
+    for col, ax in zip(cols, axes.flatten()):
+        if col in df.columns.levels[0]:
+            extent = [df[col].columns.min() - 0.5, df[col].columns.max() + 0.5,
+                      df[col].index.min() - 0.5, df[col].index.max() + 0.5]
+            im = ax.imshow(df[col] / df[col].sum(), origin='lower',
+                           extent=extent, vmin=0, vmax=1)
+        ax.set_ylabel('Called Length')
+        ax.set_xlabel('True Length')
+        plt.colorbar(im, ax=ax, label='probability')
+        ax.title.set_text(col)
+    fig.tight_layout()
+    fig.savefig(fname)
+    plt.close(fig)
+
+
+def get_errors_by_length(data, aligned_len):
+    """Calculate HP errors and Q scores separately for each length."""
+    e = data.copy()
+    e['total_err'] = e['count'] * np.abs(e['rel_len'])
+    e = e[['ref_len', 'count', 'total_err']].groupby('ref_len').sum().reset_index()
+    e['total_err_rate'] = e['total_err'] / aligned_len
+    e['total_err_to'] = np.cumsum(e['total_err'])
+    e['total_err_from'] = np.cumsum(e['total_err'].iloc[::-1])
+    e['total_err_rate_to'] = e['total_err_to'] / aligned_len
+    e['total_err_rate_from'] = e['total_err_from'] / aligned_len
+    e['QHP'] = qscore(e['total_err_rate'])
+    e['QHP_to'] = qscore(e['total_err_rate_to'])
+    e['QHP_from'] = qscore(e['total_err_rate_from'])
+    e['QHPmax'] = qscore(1. / aligned_len)
+    return e
+
+
+def plot_errors_by_length(e, fname):
+    """Plot Q score associated with HP errors as a function on HP length."""
+    fig, ax = plt.subplots()
+    e.plot('ref_len', ['QHP', 'QHP_to', 'QHP_from', 'QHPmax'],
+            ax=ax, style=['-o', '-o', '-o', ':k'],
+            label=['QHP(n)', 'QHP(<=n)', 'QHP(>=n)', 'acc. lim.'],
+            ylabel="Q score",
+            xlabel="Homopolymer length")
+    fig.savefig(fname)
+    plt.close(fig)
 
 
 def count_homopolymers(args):
@@ -140,18 +230,26 @@ def count_homopolymers(args):
         'ref_base', 'query_base', 'ref_len', 'query_len']
 
     prefix = os.path.join(args.output_dir, 'hp')
+
+    if args.bed is not None:
+        filter_trees = pomoxis.util.intervaltrees_from_bed(args.bed)
+    else:
+        filter_trees = None
+
     with open(prefix + '_catalogue.txt', 'w') as txt_fh:
         txt_fh.write('\t'.join(headers) + "\n")
-        score, counts = process_bam(args.bam, txt_fh, args.homo_len, score)
+        score, counts, aligned_length = process_bam(args.bam, txt_fh, args.homo_len, score,
+                                                    filter_trees=filter_trees)
     print("Found {} homopolymers in {}. Average score: {}".format(
         score.count, args.bam, score))
+    counts['length'] = aligned_length
     save_counts(counts, prefix + '_counts.pkl')
-    analyse_counts(counts, prefix)
+    analyse_counts(counts, prefix, aligned_length)
 
 
 def save_counts(counts, fp):
     # convert counts from nested defaultdict with factory function to plain dict
-    counts = {k: dict(v) for k,v in counts.items()}
+    counts = {k: v if type(v) == int else dict(v) for k,v in counts.items()}
     with open(fp, 'wb') as pkl_fh:
         pickle.dump(counts, pkl_fh)
 
@@ -168,8 +266,14 @@ def merge_counts(pkl_files):
     for pkl in pkl_files:
         c = load_counts(pkl)
         for base, base_counts in c.items():
-            for rlen, rlen_counts in base_counts.items():
-                counts[base][rlen].update(rlen_counts)
+            if base == 'length':
+                if 'length' not in counts:
+                    counts['length'] = base_counts
+                else:
+                    counts['length'] += base_counts
+            else:
+                for rlen, rlen_counts in base_counts.items():
+                    counts[base][rlen].update(rlen_counts)
     return counts
 
 
@@ -178,14 +282,19 @@ def analyse_homopolymers(args):
     prefix = os.path.join(args.output_dir, 'hp')
     counts = merge_counts(args.pkl)
     save_counts(counts, prefix + '_counts.pkl')
-    analyse_counts(counts, prefix)
+    if 'length' in counts:
+        aligned_length = counts['length']
+    else:
+        aligned_length = None
+    analyse_counts(counts, prefix, aligned_length)
 
 
-def analyse_counts(counts, prefix):
+def analyse_counts(counts, prefix, length=None):
     fr_correct = get_fraction_correct(counts)
     fr_correct.to_csv(prefix + '_correct_vs_len.txt', sep='\t', index=False)
     plot_fraction_correct(fr_correct, prefix + '_correct_vs_len.png')
     plot_fraction_correct(fr_correct, prefix + '_correct_vs_len_comp_pairs.png', cols=('AT', 'GC'))
+    plot_fraction_correct(fr_correct, prefix + '_correct_vs_len_all.png', cols=('ATGC',))
 
     data = get_relative_lengths(counts)
     data.to_csv(prefix + '_rel_len_counts.txt', sep='\t', index=False)
@@ -195,6 +304,19 @@ def analyse_counts(counts, prefix):
     data_comp.to_csv(prefix + '_rel_len_counts_comp_pairs.txt', sep='\t', index=False)
     plot_relative_lengths(data_comp, prefix + '_rel_len_counts_comp_pairs.png')
 
+    heatmap = get_heatmap(data, data_comp)
+    heatmap.loc[:, ['A', 'T', 'G', 'C']].to_csv(prefix + '_heatmap.txt', sep='\t')
+    heatmap.loc[:, ['AT', 'GC']].to_csv(prefix + '_heatmap_comp_pairs.txt', sep='\t')
+    heatmap.loc[:, 'ATGC'].to_csv(prefix + '_heatmap_all.txt', sep='\t')
+    plot_heatmap(heatmap, prefix + '_call_distribution.png')
+    plot_heatmap(heatmap, prefix + '_call_distribution_comp_pairs.png', cols=('AT', 'GC'))
+    plot_heatmap(heatmap, prefix + '_call_distribution_all.png', cols=('ATGC',))
+
+    if length is not None:
+        err_by_length = get_errors_by_length(data, length)
+        err_by_length.to_csv(prefix + '_qhp_vs_len.txt', sep='\t', index=False)
+        plot_errors_by_length(err_by_length, prefix + '_qhp_vs_len.png')
+
 
 def get_next_aligned_base(align, seq_start):
     next_base = seq_start
@@ -258,14 +380,21 @@ def counts_factory():
     return collections.defaultdict(collections.Counter)
 
 
-def process_bam(input_file, out_fh, homo_len, score):
+def process_bam(input_file, out_fh, homo_len, score, filter_trees=None):
     # counts[query_base][ref_len][query_len]
     counts = collections.defaultdict(counts_factory)
 
     with pysam.AlignmentFile(input_file, 'r') as bam:
+        aligned_ref_len = 0
         for seq in bam:
             if seq.is_secondary or seq.is_supplementary or seq.is_unmapped:
                 continue
+            seq_reference_start = seq.reference_start  # avoid recomputing for each HP
+            seq_reference_end = seq.reference_end  # avoid recomputing for each HP
+
+            if (filter_trees is not None and not
+                filter_trees[seq.reference_name].overlaps(seq_reference_start, seq_reference_end)):
+                continue
 
             refseq = seq.get_reference_sequence().upper()
             align_pairs = seq.get_aligned_pairs(with_seq=True)
@@ -294,6 +423,9 @@ def process_bam(input_file, out_fh, homo_len, score):
                     call_len, offset = get_longest_homopolymer(query_region, base)
                     query_start += offset
                 except ValueError:
+                    # If we are at the end of the alignment, skip this hp
+                    if align_start == 0 or align_start + ref_hp_len == len(align_pairs):
+                        continue
                     # If we don't find any matching base then we return the
                     # next query position that aligns
                     query_start = get_next_aligned_base(align_pairs, align_start)
@@ -304,11 +436,28 @@ def process_bam(input_file, out_fh, homo_len, score):
                     query_start = seq.query_length - query_start - call_len
                     base = bio.comp[base]
 
+                ref_hp_pos = seq_reference_start + ref_start
+
+                # skip hps at the end of a segment since we don't know their true length
+                if ref_hp_pos == seq_reference_start or ref_hp_pos + ref_hp_len == seq_reference_end:
+                    continue
+
+                if filter_trees is not None:
+                    # skip hps that at the end of an interval or spanning multiple intervals
+                    # since we don't know their true length
+                    interval_left_end = filter_trees[seq.reference_name][ref_hp_pos - 1]
+                    interval_right_end = filter_trees[seq.reference_name][ref_hp_pos + ref_hp_len]
+                    if (len(interval_left_end) == 0 or len(interval_right_end) == 0 or
+                        interval_left_end != interval_right_end):
+                        continue
+
+                aligned_ref_len += ref_hp_len
+
                 score += min((call_len - ref_hp_len) ** 2, 1000)
                 out_str = '\t'.join([str(i) for i in (
                     seq.query_name,
                     seq.reference_name,
-                    seq.reference_start + ref_start,
+                    ref_hp_pos,
                     '-' if seq.is_reverse else '+',
                     query_start,
                     orginal_base,
@@ -318,13 +467,13 @@ def process_bam(input_file, out_fh, homo_len, score):
                 out_fh.write(out_str + '\n')
                 # update counts of calls
                 counts[base][ref_hp_len][call_len] += 1
-    return score, counts
+    return score, counts, aligned_ref_len
 
 
 def main():
     """Entry point for homopolymer accuracy counting."""
     parser = argparse.ArgumentParser(
-        prog='homopolymer',
+        prog='assess_homopolymers',
         description='Analyse homopolymer query and reference lengths.',
         formatter_class=argparse.ArgumentDefaultsHelpFormatter)
     subparsers = parser.add_subparsers(
@@ -341,6 +490,8 @@ def main():
         help="Output directory (will be created).")
     cparser.add_argument('-l', '--homo_len', default=3, type=int,
         help='Minimum homopolymer length, default 3')
+    cparser.add_argument('-b', '--bed',
+        help='Bed file to limit search.')
 
     aparser = subparsers.add_parser('analyse',
         help='Analyse existing counts, optionally merging multiple counters.',
